<!-- Adds a new button to the admin menu that is displayed when reviewing a submission -->
<div class="col-md-12" style="margin:2px 0px 2px 0px;">
    <a href="{{ get_homepath() }}/admin/codingstyle/submission/{{ submission['_id'] }}" class="btn btn-sm btn-block btn-primary">
        <i class="fa fa-check-square fa-fw"></i> Grade coding style
    </a>
</div>