<div class="list-group mb-3">
        <a href="/submission/{{submission._id}}/codingstyle" id="coding_style" class="submission list-group-item list-group-item-action list-group-item-success">
            <i class="fa fa-star fa-fw"></i>
            Coding Style Grades
        </a>
</div>
